/*
 * Copyright 2005 - 2014  Zarafa B.V.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3, 
 * as published by the Free Software Foundation with the following additional 
 * term according to sec. 7:
 *  
 * According to sec. 7 of the GNU Affero General Public License, version
 * 3, the terms of the AGPL are supplemented with the following terms:
 * 
 * "Zarafa" is a registered trademark of Zarafa B.V. The licensing of
 * the Program under the AGPL does not imply a trademark license.
 * Therefore any rights, title and interest in our trademarks remain
 * entirely with us.
 * 
 * However, if you propagate an unmodified version of the Program you are
 * allowed to use the term "Zarafa" to indicate that you distribute the
 * Program. Furthermore you may use our trademarks where it is necessary
 * to indicate the intended purpose of a product or service provided you
 * use it in accordance with honest practices in industrial or commercial
 * matters.  If you want to propagate modified versions of the Program
 * under the name "Zarafa" or "Zarafa Server", you may only do so if you
 * have a written permission by Zarafa B.V. (to acquire a permission
 * please contact Zarafa at trademark@zarafa.com).
 * 
 * The interactive user interface of the software displays an attribution
 * notice containing the term "Zarafa" and/or the logo of Zarafa.
 * Interactive user interfaces of unmodified and modified versions must
 * display Appropriate Legal Notices according to sec. 5 of the GNU
 * Affero General Public License, version 3, when you propagate
 * unmodified or modified versions of the Program. In accordance with
 * sec. 7 b) of the GNU Affero General Public License, version 3, these
 * Appropriate Legal Notices must retain the logo of Zarafa or display
 * the words "Initial Development by Zarafa" if the display of the logo
 * is not reasonably feasible for technical reasons. The use of the logo
 * of Zarafa in Legal Notices is allowed for unmodified and modified
 * versions of the software.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *  
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include "platform.h"
#include <map>
#include <string>
#include <boost/assign.hpp>
namespace ba = boost::assign;

// map olson name to windows name
std::map<std::string, std::wstring> mapWindowsOlsonNames = ba::map_list_of
	("Atlantic/Canary", L"GMT Standard Time")
	("Europe/Lisbon", L"GMT Standard Time")
	("Antarctica/Mawson", L"West Asia Standard Time")
	("Etc/GMT+3", L"SA Eastern Standard Time")
	("Etc/GMT+2", L"UTC-02")
	("Etc/GMT+1", L"Cape Verde Standard Time")
	("Etc/GMT+7", L"US Mountain Standard Time")
	("Etc/GMT+6", L"Central America Standard Time")
	("Etc/GMT+5", L"SA Pacific Standard Time")
	("Etc/GMT+4", L"SA Western Standard Time")
	("Pacific/Wallis", L"UTC+12")
	("Europe/Skopje", L"Central European Standard Time")
	("America/Coral_Harbour", L"SA Pacific Standard Time")
	("Asia/Dhaka", L"Bangladesh Standard Time")
	("America/St_Lucia", L"SA Western Standard Time")
	("Asia/Kashgar", L"China Standard Time")
	("America/Phoenix", L"US Mountain Standard Time")
	("Asia/Kuwait", L"Arab Standard Time")
	("America/Nipigon", L"Eastern Standard Time")
	("Asia/Hong_Kong", L"China Standard Time")
	("Arctic/Longyearbyen", L"W. Europe Standard Time")
	("Europe/Guernsey", L"GMT Standard Time")
	("Europe/Paris", L"Romance Standard Time")
	("Europe/Stockholm", L"W. Europe Standard Time")
	("Pacific/Fiji", L"Fiji Standard Time")
	("Pacific/Apia", L"Samoa Standard Time")
	("Pacific/Pago_Pago", L"UTC-11")
	("Asia/Rangoon", L"Myanmar Standard Time")
	("America/Mexico_City", L"Central Standard Time (Mexico)")
	("America/Puerto_Rico", L"SA Western Standard Time")
	("Pacific/Midway", L"UTC-11")
	("Europe/Berlin", L"W. Europe Standard Time")
	("Europe/Zurich", L"W. Europe Standard Time")
	("America/Belem", L"SA Eastern Standard Time")
	("Antarctica/Davis", L"SE Asia Standard Time")
	("Asia/Krasnoyarsk", L"North Asia Standard Time")
	("Atlantic/Bermuda", L"Atlantic Standard Time")
	("America/Araguaina", L"E. South America Standard Time")
	("Asia/Tehran", L"Iran Standard Time")
	("Asia/Saigon", L"SE Asia Standard Time")
	("Asia/Baku", L"Azerbaijan Standard Time")
	("America/St_Barthelemy", L"SA Western Standard Time")
	("America/Santarem", L"SA Eastern Standard Time")
	("America/Danmarkshavn", L"UTC")
	("America/Scoresbysund", L"Azores Standard Time")
	("America/Eirunepe", L"SA Western Standard Time")
	("America/Caracas", L"Venezuela Standard Time")
	("Asia/Baghdad", L"Arabic Standard Time")
	("Africa/Monrovia", L"Greenwich Standard Time")
	("America/St_Vincent", L"SA Western Standard Time")
	("America/Vancouver", L"Pacific Standard Time")
	("Asia/Thimphu", L"Bangladesh Standard Time")
	("Africa/Accra", L"Greenwich Standard Time")
	("America/Belize", L"Central America Standard Time")
	("America/Port_of_Spain", L"SA Western Standard Time")
	("Asia/Tashkent", L"West Asia Standard Time")
	("Asia/Tokyo", L"Tokyo Standard Time")
	("Australia/Sydney", L"AUS Eastern Standard Time")
	("Europe/Riga", L"FLE Standard Time")
	("Asia/Dili", L"Tokyo Standard Time")
	("Africa/Mbabane", L"South Africa Standard Time")
	("Asia/Oral", L"West Asia Standard Time")
	("Asia/Aden", L"Arab Standard Time")
	("Europe/Isle_of_Man", L"GMT Standard Time")
	("Europe/Istanbul", L"Turkey Standard Time")
	("Europe/Mariehamn", L"FLE Standard Time")
	("Australia/Lindeman", L"E. Australia Standard Time")
	("Pacific/Galapagos", L"Central America Standard Time")
	("America/Bogota", L"SA Pacific Standard Time")
	("America/Dawson", L"Pacific Standard Time")
	("America/Chicago", L"Central Standard Time")
	("Pacific/Kwajalein", L"UTC+12")
	("Australia/Broken_Hill", L"Cen. Australia Standard Time")
	("America/Cuiaba", L"Central Brazilian Standard Time")
	("Indian/Christmas", L"SE Asia Standard Time")
	("Asia/Vladivostok", L"Vladivostok Standard Time")
	("Europe/Brussels", L"Romance Standard Time")
	("Asia/Chongqing", L"China Standard Time")
	("America/Noronha", L"UTC-02")
	("Europe/Madrid", L"Romance Standard Time")
	("Africa/Algiers", L"W. Central Africa Standard Time")
	("Africa/Harare", L"South Africa Standard Time")
	("Africa/Ndjamena", L"W. Central Africa Standard Time")
	("America/Costa_Rica", L"Central America Standard Time")
	("Europe/Ljubljana", L"Central Europe Standard Time")
	("Indian/Mayotte", L"E. Africa Standard Time")
	("Asia/Phnom_Penh", L"SE Asia Standard Time")
	("America/Managua", L"Central America Standard Time")
	("America/Pangnirtung", L"Eastern Standard Time")
	("Asia/Nicosia", L"E. Europe Standard Time")
	("America/Tijuana", L"Pacific Standard Time")
	("Pacific/Fakaofo", L"Tonga Standard Time")
	("America/Martinique", L"SA Western Standard Time")
	("America/Antigua", L"SA Western Standard Time")
	("America/Argentina/La_Rioja", L"Argentina Standard Time")
	("Pacific/Tahiti", L"Hawaiian Standard Time")
	("Asia/Brunei", L"Singapore Standard Time")
	("Europe/Zagreb", L"Central European Standard Time")
	("America/Asuncion", L"Paraguay Standard Time")
	("Europe/Vienna", L"W. Europe Standard Time")
	("Australia/Hobart", L"Tasmania Standard Time")
	("America/Juneau", L"Alaskan Standard Time")
	("America/Inuvik", L"Mountain Standard Time")
	("America/Ojinaga", L"Mountain Standard Time")
	("America/Montreal", L"Eastern Standard Time")
	("Asia/Seoul", L"Korea Standard Time")
	("Indian/Comoro", L"E. Africa Standard Time")
	("Antarctica/Rothera", L"SA Eastern Standard Time")
	("Europe/Tallinn", L"FLE Standard Time")
	("Indian/Mahe", L"Mauritius Standard Time")
	("Asia/Calcutta", L"India Standard Time")
	("America/Creston", L"US Mountain Standard Time")
	("Asia/Singapore", L"Singapore Standard Time")
	("Africa/Nairobi", L"E. Africa Standard Time")
	("Asia/Pontianak", L"SE Asia Standard Time")
	("America/Maceio", L"SA Eastern Standard Time")
	("Africa/Cairo", L"Egypt Standard Time")
	("Europe/Moscow", L"Russian Standard Time")
	("Asia/Pyongyang", L"Korea Standard Time")
	("Asia/Ulaanbaatar", L"Ulaanbaatar Standard Time")
	("America/Rainy_River", L"Central Standard Time")
	("Africa/Kampala", L"E. Africa Standard Time")
	("Asia/Colombo", L"Sri Lanka Standard Time")
	("Australia/Adelaide", L"Cen. Australia Standard Time")
	("America/Cambridge_Bay", L"Mountain Standard Time")
	("Africa/Luanda", L"W. Central Africa Standard Time")
	("America/Indiana/Winamac", L"Eastern Standard Time")
	("America/Cordoba", L"Argentina Standard Time")
	("Asia/Tbilisi", L"Georgian Standard Time")
	("Europe/Gibraltar", L"W. Europe Standard Time")
	("Asia/Karachi", L"Pakistan Standard Time")
	("Asia/Harbin", L"China Standard Time")
	("Etc/GMT-9", L"Tokyo Standard Time")
	("Etc/GMT-8", L"Singapore Standard Time")
	("America/Bahia_Banderas", L"Central Standard Time (Mexico)")
	("Etc/GMT-1", L"W. Central Africa Standard Time")
	("Etc/GMT-3", L"E. Africa Standard Time")
	("Etc/GMT-2", L"South Africa Standard Time")
	("Etc/GMT-5", L"West Asia Standard Time")
	("Etc/GMT-4", L"Arabian Standard Time")
	("Etc/GMT-7", L"SE Asia Standard Time")
	("Etc/GMT-6", L"Central Asia Standard Time")
	("America/Boa_Vista", L"SA Western Standard Time")
	("Africa/Tripoli", L"W. Europe Standard Time")
	("Indian/Reunion", L"Mauritius Standard Time")
	("Atlantic/Stanley", L"SA Eastern Standard Time")
	("Etc/GMT-13", L"Tonga Standard Time")
	("America/Blanc-Sablon", L"SA Western Standard Time")
	("America/Santo_Domingo", L"SA Western Standard Time")
	("Antarctica/Syowa", L"E. Africa Standard Time")
	("America/Jamaica", L"SA Pacific Standard Time")
	("Europe/Kiev", L"FLE Standard Time")
	("Europe/Budapest", L"Central Europe Standard Time")
	("Indian/Mauritius", L"Mauritius Standard Time")
	("America/Goose_Bay", L"Atlantic Standard Time")
	("Asia/Amman", L"Jordan Standard Time")
	("Asia/Sakhalin", L"Vladivostok Standard Time")
	("Africa/Windhoek", L"Namibia Standard Time")
	("America/Sitka", L"Alaskan Standard Time")
	("Asia/Katmandu", L"Nepal Standard Time")
	("America/Guyana", L"SA Western Standard Time")
	("America/Sao_Paulo", L"E. South America Standard Time")
	("America/Lower_Princes", L"SA Western Standard Time")
	("Australia/Perth", L"W. Australia Standard Time")
	("Africa/Djibouti", L"E. Africa Standard Time")
	("Asia/Jakarta", L"SE Asia Standard Time")
	("Antarctica/Palmer", L"Pacific SA Standard Time")
	("EST5EDT", L"Eastern Standard Time")
	("Africa/Johannesburg", L"South Africa Standard Time")
	("Asia/Irkutsk", L"North Asia East Standard Time")
	("Africa/Niamey", L"W. Central Africa Standard Time")
	("Pacific/Kosrae", L"Central Pacific Standard Time")
	("Africa/Casablanca", L"Morocco Standard Time")
	("Asia/Bishkek", L"Central Asia Standard Time")
	("Africa/Nouakchott", L"Greenwich Standard Time")
	("Europe/Vilnius", L"FLE Standard Time")
	("America/Cayenne", L"SA Eastern Standard Time")
	("Africa/Mogadishu", L"E. Africa Standard Time")
	("America/Kentucky/Monticello", L"Eastern Standard Time")
	("America/Rio_Branco", L"SA Western Standard Time")
	("America/Cancun", L"Central Standard Time (Mexico)")
	("Pacific/Guam", L"West Pacific Standard Time")
	("Antarctica/Macquarie", L"Central Pacific Standard Time")
	("Atlantic/Azores", L"Azores Standard Time")
	("Asia/Shanghai", L"China Standard Time")
	("America/Godthab", L"Greenland Standard Time")
	("Asia/Beirut", L"Middle East Standard Time")
	("Africa/Maputo", L"South Africa Standard Time")
	("Asia/Bahrain", L"Arab Standard Time")
	("Asia/Ashgabat", L"West Asia Standard Time")
	("Asia/Riyadh", L"Arab Standard Time")
	("Atlantic/Faeroe", L"GMT Standard Time")
	("America/Monterrey", L"Central Standard Time (Mexico)")
	("America/Anguilla", L"SA Western Standard Time")
	("Asia/Damascus", L"Syria Standard Time")
	("America/North_Dakota/Center", L"Central Standard Time")
	("America/Indiana/Vevay", L"US Eastern Standard Time")
	("Atlantic/St_Helena", L"Greenwich Standard Time")
	("America/Barbados", L"SA Western Standard Time")
	("Europe/Vatican", L"W. Europe Standard Time")
	("America/Indiana/Vincennes", L"Eastern Standard Time")
	("Asia/Almaty", L"Central Asia Standard Time")
	("Africa/Lome", L"Greenwich Standard Time")
	("Africa/Brazzaville", L"W. Central Africa Standard Time")
	("America/Nome", L"Alaskan Standard Time")
	("Europe/Dublin", L"GMT Standard Time")
	("America/Yakutat", L"Alaskan Standard Time")
	("America/Tortola", L"SA Western Standard Time")
	("Europe/Vaduz", L"W. Europe Standard Time")
	("Africa/Asmera", L"E. Africa Standard Time")
	("Europe/Minsk", L"Kaliningrad Standard Time")
	("America/Buenos_Aires", L"Argentina Standard Time")
	("Africa/Maseru", L"South Africa Standard Time")
	("America/St_Johns", L"Newfoundland Standard Time")
	("America/Santa_Isabel", L"Pacific Standard Time (Mexico)")
	("Asia/Kuching", L"Singapore Standard Time")
	("Africa/Libreville", L"W. Central Africa Standard Time")
	("Africa/Freetown", L"Greenwich Standard Time")
	("Africa/Bissau", L"Greenwich Standard Time")
	("Europe/Samara", L"Russian Standard Time")
	("Europe/Amsterdam", L"W. Europe Standard Time")
	("Europe/Tirane", L"Central Europe Standard Time")
	("Pacific/Saipan", L"West Pacific Standard Time")
	("Africa/Abidjan", L"Greenwich Standard Time")
	("Europe/Zaporozhye", L"FLE Standard Time")
	("America/El_Salvador", L"Central America Standard Time")
	("Europe/Podgorica", L"Central Europe Standard Time")
	("Africa/Juba", L"E. Africa Standard Time")
	("America/Santiago", L"Pacific SA Standard Time")
	("America/Argentina/San_Luis", L"Argentina Standard Time")
	("America/Aruba", L"SA Western Standard Time")
	("America/Indianapolis", L"US Eastern Standard Time")
	("America/Regina", L"Canada Central Standard Time")
	("Pacific/Truk", L"West Pacific Standard Time")
	("Pacific/Funafuti", L"UTC+12")
	("America/Merida", L"Central Standard Time (Mexico)")
	("America/Guatemala", L"Central America Standard Time")
	("Asia/Magadan", L"Magadan Standard Time")
	("Africa/Sao_Tome", L"Greenwich Standard Time")
	("Asia/Makassar", L"Singapore Standard Time")
	("Africa/Bujumbura", L"South Africa Standard Time")
	("Europe/Chisinau", L"GTB Standard Time")
	("Europe/Warsaw", L"Central European Standard Time")
	("Asia/Yekaterinburg", L"Ekaterinburg Standard Time")
	("Antarctica/Casey", L"W. Australia Standard Time")
	("Pacific/Enderbury", L"Tonga Standard Time")
	("America/Thule", L"Atlantic Standard Time")
	("America/Louisville", L"Eastern Standard Time")
	("America/Moncton", L"Atlantic Standard Time")
	("Europe/Helsinki", L"FLE Standard Time")
	("Atlantic/Cape_Verde", L"Cape Verde Standard Time")
	("America/Tegucigalpa", L"Central America Standard Time")
	("Indian/Cocos", L"Myanmar Standard Time")
	("America/Boise", L"Mountain Standard Time")
	("America/Guadeloupe", L"SA Western Standard Time")
	("America/Nassau", L"Eastern Standard Time")
	("Europe/Prague", L"Central Europe Standard Time")
	("America/Halifax", L"Atlantic Standard Time")
	("Asia/Hovd", L"SE Asia Standard Time")
	("America/Manaus", L"SA Western Standard Time")
	("America/Rankin_Inlet", L"Central Standard Time")
	("America/Catamarca", L"Argentina Standard Time")
	("America/North_Dakota/Beulah", L"Central Standard Time")
	("America/Chihuahua", L"Mountain Standard Time (Mexico)")
	("America/Iqaluit", L"Eastern Standard Time")
	("America/Argentina/Rio_Gallegos", L"Argentina Standard Time")
	("Europe/Volgograd", L"Russian Standard Time")
	("Africa/Bamako", L"Greenwich Standard Time")
	("Asia/Novokuznetsk", L"N. Central Asia Standard Time")
	("Europe/Uzhgorod", L"FLE Standard Time")
	("Africa/Banjul", L"Greenwich Standard Time")
	("Asia/Aqtau", L"West Asia Standard Time")
	("Pacific/Palau", L"Tokyo Standard Time")
	("Africa/Malabo", L"W. Central Africa Standard Time")
	("PST8PDT", L"Pacific Standard Time")
	("Atlantic/Madeira", L"GMT Standard Time")
	("Pacific/Noumea", L"Central Pacific Standard Time")
	("Africa/Kinshasa", L"W. Central Africa Standard Time")
	("Europe/Malta", L"W. Europe Standard Time")
	("America/Argentina/Ushuaia", L"Argentina Standard Time")
	("Asia/Bangkok", L"SE Asia Standard Time")
	("Pacific/Niue", L"UTC-11")
	("Australia/Brisbane", L"E. Australia Standard Time")
	("America/Recife", L"SA Eastern Standard Time")
	("Asia/Yerevan", L"Caucasus Standard Time")
	("America/La_Paz", L"SA Western Standard Time")
	("Asia/Urumqi", L"China Standard Time")
	("Africa/Lusaka", L"South Africa Standard Time")
	("Pacific/Guadalcanal", L"Central Pacific Standard Time")
	("America/Yellowknife", L"Mountain Standard Time")
	("Asia/Vientiane", L"SE Asia Standard Time")
	("Europe/Kaliningrad", L"Kaliningrad Standard Time")
	("Africa/Conakry", L"Greenwich Standard Time")
	("America/Argentina/Tucuman", L"Argentina Standard Time")
	("Europe/Oslo", L"W. Europe Standard Time")
	("Australia/Currie", L"Tasmania Standard Time")
	("America/St_Kitts", L"SA Western Standard Time")
	("America/Panama", L"SA Pacific Standard Time")
	("America/Hermosillo", L"US Mountain Standard Time")
	("Asia/Hebron", L"Egypt Standard Time")
	("America/Guayaquil", L"SA Pacific Standard Time")
	("Asia/Kuala_Lumpur", L"Singapore Standard Time")
	("Europe/London", L"GMT Standard Time")
	("America/Menominee", L"Central Standard Time")
	("Asia/Kamchatka", L"Magadan Standard Time")
	("Asia/Jayapura", L"Tokyo Standard Time")
	("America/Matamoros", L"Central Standard Time")
	("Asia/Qatar", L"Arab Standard Time")
	("Asia/Dushanbe", L"West Asia Standard Time")
	("Asia/Yakutsk", L"Yakutsk Standard Time")
	("Asia/Omsk", L"N. Central Asia Standard Time")
	("Africa/Bangui", L"W. Central Africa Standard Time")
	("America/Paramaribo", L"SA Eastern Standard Time")
	("Etc/GMT-11", L"Central Pacific Standard Time")
	("Etc/GMT-10", L"West Pacific Standard Time")
	("Africa/Lubumbashi", L"South Africa Standard Time")
	("Etc/GMT-12", L"UTC+12")
	("Europe/Bratislava", L"Central Europe Standard Time")
	("Asia/Anadyr", L"Magadan Standard Time")
	("America/New_York", L"Eastern Standard Time")
	("CST6CDT", L"Central Standard Time")
	("Pacific/Rarotonga", L"Hawaiian Standard Time")
	("America/Dominica", L"SA Western Standard Time")
	("Africa/Porto-Novo", L"W. Central Africa Standard Time")
	("Asia/Samarkand", L"West Asia Standard Time")
	("America/Toronto", L"Eastern Standard Time")
	("America/Bahia", L"Bahia Standard Time")
	("Indian/Maldives", L"West Asia Standard Time")
	("Africa/Ouagadougou", L"Greenwich Standard Time")
	("Antarctica/South_Pole", L"New Zealand Standard Time")
	("Asia/Muscat", L"Arabian Standard Time")
	("America/Edmonton", L"Mountain Standard Time")
	("Pacific/Wake", L"UTC+12")
	("America/Indiana/Tell_City", L"Central Standard Time")
	("Australia/Darwin", L"AUS Central Standard Time")
	("America/Whitehorse", L"Pacific Standard Time")
	("America/Swift_Current", L"Canada Central Standard Time")
	("Europe/Copenhagen", L"Romance Standard Time")
	("America/Argentina/Salta", L"Argentina Standard Time")
	("America/Montserrat", L"SA Western Standard Time")
	("America/Mendoza", L"Argentina Standard Time")
	("Europe/Simferopol", L"FLE Standard Time")
	("Africa/Blantyre", L"South Africa Standard Time")
	("America/Detroit", L"Eastern Standard Time")
	("America/Shiprock", L"Mountain Standard Time")
	("America/Grenada", L"SA Western Standard Time")
	("America/Indiana/Petersburg", L"Eastern Standard Time")
	("Asia/Dubai", L"Arabian Standard Time")
	("Europe/Athens", L"GTB Standard Time")
	("America/Port-au-Prince", L"SA Pacific Standard Time")
	("America/Cayman", L"SA Pacific Standard Time")
	("Africa/Dar_es_Salaam", L"E. Africa Standard Time")
	("America/Curacao", L"SA Western Standard Time")
	("Indian/Kerguelen", L"West Asia Standard Time")
	("Africa/Khartoum", L"E. Africa Standard Time")
	("Asia/Manila", L"Singapore Standard Time")
	("Europe/Sarajevo", L"Central European Standard Time")
	("America/Jujuy", L"Argentina Standard Time")
	("Africa/Douala", L"W. Central Africa Standard Time")
	("Europe/Rome", L"W. Europe Standard Time")
	("America/Argentina/San_Juan", L"Argentina Standard Time")
	("America/North_Dakota/New_Salem", L"Central Standard Time")
	("America/Kralendijk", L"SA Western Standard Time")
	("Pacific/Port_Moresby", L"West Pacific Standard Time")
	("Europe/Jersey", L"GMT Standard Time")
	("Europe/Andorra", L"W. Europe Standard Time")
	("Europe/Luxembourg", L"W. Europe Standard Time")
	("Pacific/Honolulu", L"Hawaiian Standard Time")
	("America/St_Thomas", L"SA Western Standard Time")
	("Pacific/Majuro", L"UTC+12")
	("America/Mazatlan", L"Mountain Standard Time (Mexico)")
	("Asia/Macau", L"China Standard Time")
	("MST7MDT", L"Mountain Standard Time")
	("Europe/Belgrade", L"Central Europe Standard Time")
	("Asia/Choibalsan", L"Ulaanbaatar Standard Time")
	("Antarctica/McMurdo", L"New Zealand Standard Time")
	("America/Thunder_Bay", L"Eastern Standard Time")
	("America/Los_Angeles", L"Pacific Standard Time")
	("Asia/Kabul", L"Afghanistan Standard Time")
	("Indian/Antananarivo", L"E. Africa Standard Time")
	("Atlantic/Reykjavik", L"Greenwich Standard Time")
	("Etc/GMT+12", L"Dateline Standard Time")
	("Etc/GMT+11", L"UTC-11")
	("Etc/GMT+10", L"Hawaiian Standard Time")
	("Pacific/Ponape", L"Central Pacific Standard Time")
	("Pacific/Tongatapu", L"Tonga Standard Time")
	("America/Marigot", L"SA Western Standard Time")
	("Atlantic/South_Georgia", L"UTC-02")
	("Africa/El_Aaiun", L"Greenwich Standard Time")
	("America/Campo_Grande", L"Central Brazilian Standard Time")
	("America/Dawson_Creek", L"US Mountain Standard Time")
	("Antarctica/Vostok", L"Central Asia Standard Time")
	("Europe/Bucharest", L"GTB Standard Time")
	("America/Porto_Velho", L"SA Western Standard Time")
	("Europe/Monaco", L"W. Europe Standard Time")
	("America/Indiana/Marengo", L"US Eastern Standard Time")
	("Africa/Ceuta", L"Romance Standard Time")
	("America/Winnipeg", L"Central Standard Time")
	("Asia/Aqtobe", L"West Asia Standard Time")
	("Africa/Dakar", L"Greenwich Standard Time")
	("America/Fortaleza", L"SA Eastern Standard Time")
	("Pacific/Tarawa", L"UTC+12")
	("Africa/Addis_Ababa", L"E. Africa Standard Time")
	("Pacific/Efate", L"Central Pacific Standard Time")
	("Pacific/Johnston", L"Hawaiian Standard Time")
	("Asia/Qyzylorda", L"Central Asia Standard Time")
	("Europe/San_Marino", L"W. Europe Standard Time")
	("Asia/Jerusalem", L"Israel Standard Time")
	("Pacific/Auckland", L"New Zealand Standard Time")
	("America/Denver", L"Mountain Standard Time")
	("Indian/Chagos", L"Central Asia Standard Time")
	("America/Glace_Bay", L"Atlantic Standard Time")
	("Africa/Gaborone", L"South Africa Standard Time")
	("Africa/Tunis", L"W. Central Africa Standard Time")
	("America/Montevideo", L"Montevideo Standard Time")
	("America/Resolute", L"Central Standard Time")
	("Australia/Melbourne", L"AUS Eastern Standard Time")
	("Etc/GMT", L"UTC")
	("Asia/Gaza", L"Egypt Standard Time")
	("Asia/Taipei", L"Taipei Standard Time")
	("Antarctica/DumontDUrville", L"West Pacific Standard Time")
	("America/Indiana/Knox", L"Central Standard Time")
	("Asia/Novosibirsk", L"N. Central Asia Standard Time")
	("Africa/Kigali", L"South Africa Standard Time")
	("America/Grand_Turk", L"Eastern Standard Time")
	("Africa/Lagos", L"W. Central Africa Standard Time")
	("Europe/Sofia", L"FLE Standard Time")
	("America/Lima", L"SA Pacific Standard Time")
	("America/Anchorage", L"Alaskan Standard Time")
	("Pacific/Nauru", L"UTC+12")
;
